import util
import os

import defaults
import log

def runCompiled(program, args):
    return ["./" + util.replaceSuffix(program, defaults.exe)] + args

def noCompile(program):
    return 0

def joinClasspath(classpath):
    return ["-cp", (";" if util.windows else ":").join(classpath)] if classpath else []

def runJar(jar, args = []):
    return java(args = ["-jar", jar] + args)


def java(classpath = [], args = []):
    return [os.path.join(javaHome(), "java")] + defaults.javaOptions + joinClasspath(classpath) + list(args)


myJavaHome = None
def javaHome():
    global myJavaHome
    if myJavaHome is None:
        myJavaHome = ""
        if os.name == "nt":
            if "JAVA_HOME" in os.environ:
                myJavaHome = os.environ["JAVA_HOME"] + "\\bin\\"
            else:
                log.warning("Warining: JAVA_HOME environment variable not set")
    return myJavaHome


def jar(*args):
    return util.execute(os.path.join(javaHome(), "jar"), *args)


def extractJar(file, dir):
    cwd = os.getcwd()
    os.chdir(dir)
    jar("xf", file)
    os.chdir(cwd)


def javaToJar(program):
    name = util.deleteSuffix(program)
    temp = "__" + name
    util.removeDirs(temp)
    try:
        util.removeFiles(name + ".jar")
        util.mkdirs(temp)
        if name == "Check":
            extractJar(util.relpath("testlib/testlib4j.jar"), temp)
            manifest = {
                "Main-Class": "ru.ifmo.testlib.CheckerFramework",
                "Checker-Class": "Check"
            }
        elif name == "Validate" and os.path.exists(util.relpath("testlib/validate4j.jar")):
            extractJar(util.relpath("testlib/validate4j.jar"), temp)
            manifest = {
                "Main-Class": "ru.ifmo.validate.ValidatorFramework",
                "Checker-Class": "Validate"
            }
        else:
            manifest = {"Main-Class": name}

        manifest["Class-Path"] = " ".join(javaLibs())

        return util.execute(*[
                javaHome() + "javac",
                "-sourcepath", ".",
                "-d", temp,
            ] + defaults.javacOptions + [
            ] + joinClasspath(defaults.javaClasspath) + [
                program
            ]
        ) or createJar(name, temp, manifest)
    finally:
        util.removeDirs(temp)


myKotlinHome = None
def kotlinHome():
    global myKotlinHome
    if myKotlinHome is None:
        myKotlinHome = ()
        if "KOTLIN_HOME" in os.environ:
            myKotlinHome = os.path.join(os.environ["KOTLIN_HOME"], "lib")
        else:
            myKotlinHome = util.relpath("p/kotlin/lib")
    return myKotlinHome


def kotlinToJar(program):
    def jarFile(name):
        return os.path.join(kotlinHome(), name)

    name = util.deleteSuffix(program)
    temp = "__" + name
    util.removeDirs(temp)
    try:
        util.removeFiles(name + ".jar")
        util.mkdirs(temp)
        extractJar(jarFile("kotlin-runtime.jar"), temp)
        util.execute(*java(defaults.javaClasspath, [
            "-jar", jarFile("kotlin-compiler.jar"),
            "-d", temp,
            program
        ]))
        mainClass = name[:1].upper() + name[1:] + "Kt"
        if not os.path.isfile(temp + "/" + mainClass + ".class"):
            if not os.path.isfile(temp + "/" + name + "/" + mainClass + ".class"):
                raise Exception("Cannot find main class " + mainClass)
            mainClass = name + "." + mainClass
        return createJar(name, temp, {
            "Main-Class": mainClass,
            "Class-Path": " ".join(javaLibs())
        })
    finally:
        util.removeDirs(temp)


def javaLibs():
    return ["lib/" + file for file in os.listdir("lib/") if file.endswith(".jar")] if os.path.isdir("lib/") else []


def createJar(name, dir, manifest):
    util.mkdirs(os.path.join(dir, "META-INF"))
    manifestFile = os.path.join(dir, "META-INF/MANIFEST.MF")
    util.writeFile(manifestFile, "{}\n", "\n".join([k + ": " + v for (k, v) in manifest.items()]))
    return jar(
        "cfm", name + ".jar", manifestFile,
        "-C", dir,
        "."
    )


def addArgs(f):
    return (lambda program, args: f(program) + args)

def substituteProgram(program, command):
    return [
        c.format(program = program, programNoSuffix = util.deleteSuffix(program))
        for c in command
    ]

def compiled(suffix, description, comments, compileCommand, run = runCompiled, tempSuffixes = []):
    def compile(program):
        try:
            if util.windows:
                util.execute("reg.exe", "add", "HKEY_LOCAL_MACHINE\\SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Image File Execution Options\\" + util.replaceSuffix(program, defaults.exe), "/f")
                print(util.replaceSuffix(program, defaults.exe))
            return util.execute(*substituteProgram(program, compileCommand))
        finally:
            util.removeFiles(*[util.replaceSuffix(program, suffix) for suffix in tempSuffixes])

    return language(suffix, description, comments, compile, run)

def compiledWinLin(suffix, description, comments, compileCommandWin, compileCommandLin, run = runCompiled, tempSuffixes = []):
    return compiled(
        suffix,
        description,
        comments,
        compileCommandWin if util.windows else compileCommandLin,
        run,
        tempSuffixes
    )

def script(suffix, description, comments, runCommand):
    return language(suffix, description, comments, noCompile,
        addArgs(lambda program: substituteProgram(program, runCommand)))

def language(suffix, description, comments, compile, runCommand):
    return dict(
        suffix = suffix,
        description = description,
        comments = comments,
        compile = compile,
        run = runCommand
    )

def comments(begin, line, end):
    return dict(begin = begin, line = line, end = end)

comments_c = comments("/*", "*", "*/")
comments_pascal = comments("(*", "*", "*)")
comments_haskell = comments("{-", "-", "-}")
comments_xml = comments("<!--", "=", "-->")
comments_basic = comments("'", "=", "'")
comments_python = comments("#", "#", "#")
