/******************************************************************/
/* ACM ICPC 2017-2018, NEERC                                      */
/* Northern Subregional Contest                                   */
/* St Petersburg, November 4, 2017                                */
/******************************************************************/
/* Problem D. Dividing Marbles                                    */
/*                                                                */
/* Original idea         Gennady Korotkevich                      */
/* Problem statement     Gennady Korotkevich                      */
/* Test set              Gennady Korotkevich                      */
/******************************************************************/
/* Tests                                                          */
/*                                                                */
/* Author                Gennady Korotkevich                      */
/******************************************************************/

import java.io.*;
import java.nio.file.*;
import java.util.*;

public class Tests {
    
    private static class SingleTest {
        int[] d;

        public SingleTest(int[] dd) {
            d = dd;
        }

        public SingleTest(int d0, int d1, int d2, int d3) {
            d = new int[]{d0, d1, d2, d3};
        }
    }

    private static int nTests = 0;
    private static final Path root = Paths.get("tests");
    private static int MAX_T = 500;
    private static int MAX_N = 20;
    private static Set<Integer> used = new HashSet<>();

    private static final Random random = new Random(2347238703487348571L);

    private static boolean shuffle = false;

    public static void main(String[] args) throws IOException {
        Files.createDirectories(root);
        // sample
        test(new SingleTest[] {new SingleTest(1, 0, 1, 0), new SingleTest(0, 1, 2, 3)});
        // all possible cases
        shuffle = true;
        {
            List<SingleTest> tests = new ArrayList<>();
            for (int d0 = 0; d0 <= MAX_N; d0++) {
                for (int d1 = d0; d1 <= MAX_N; d1++) {
                    for (int d2 = d1; d2 <= MAX_N; d2++) {
                        for (int d3 = d2; d3 <= MAX_N; d3++) {
                            tests.add(new SingleTest(d0, d1, d2, d3));
                        }
                    }
                }
            }
            Collections.shuffle(tests, random);
            int ptr = 0;
            while (ptr < tests.size()) {
                int to = Math.min(ptr + MAX_T, tests.size());
                SingleTest[] curTests = new SingleTest[to - ptr];
                for (int i = ptr; i < to; i++) {
                    curTests[i - ptr] = tests.get(i);
                }
                test(curTests);
                ptr += MAX_T;
            }
        }
        // some maximum cases
        {
            List<SingleTest> tests = new ArrayList<>();
            for (int d0 = 0; d0 < MAX_N; d0++) {
                for (int d1 = d0 + 1; d1 < MAX_N; d1++) {
                    for (int d2 = d1 + 1; d2 < MAX_N; d2++) {
                        int d3 = MAX_N;
                        tests.add(new SingleTest(d0, d1, d2, d3));
                    }
                }
            }
            Collections.shuffle(tests, random);
            int ptr = 0;
            while (ptr < tests.size()) {
                int to = Math.min(ptr + MAX_T, tests.size());
                SingleTest[] curTests = new SingleTest[to - ptr];
                for (int i = ptr; i < to; i++) {
                    curTests[i - ptr] = tests.get(i);
                }
                test(curTests);
                ptr += MAX_T;
            }
        }
        {
            SingleTest[] curTests = new SingleTest[MAX_T];
            for (int i = 0; i < MAX_T; i++) {
                curTests[i] = new SingleTest(MAX_N, MAX_N - 1, MAX_N - 2, MAX_N - 3);
            }
            test(curTests);
        }
        {
            SingleTest[] curTests = new SingleTest[MAX_T];
            for (int i = 0; i < MAX_T; i++) {
                curTests[i] = new SingleTest(MAX_N, MAX_N - 1, MAX_N - 2, 0);
            }
            test(curTests);
        }
        {
            SingleTest[] curTests = new SingleTest[MAX_T];
            for (int i = 0; i < MAX_T; i++) {
                int a, b;
                do {
                    a = random.nextInt(MAX_N - 1) + 1;
                    b = random.nextInt(MAX_N - 1) + 1;
                } while (a == b);
                curTests[i] = new SingleTest(MAX_N, a, b, 0);
            }
            test(curTests);
        }
    }

    private static void test(SingleTest[] tests) {
        int cnt = tests.length;
        for (int i = 0; i < cnt; i++) {
            for (int j = 0; j < 4; j++) {
                if (tests[i].d[j] < 0 || tests[i].d[j] > MAX_N) {
                    throw new AssertionError("Bad number " + tests[i].d[j]);
                }
            }
        }
        if (shuffle) {
            for (int i = 0; i < cnt; i++) {
                ArrayList<Integer> d = new ArrayList<>();
                d.add(tests[i].d[0]);
                d.add(tests[i].d[1]);
                d.add(tests[i].d[2]);
                d.add(tests[i].d[3]);
                Collections.shuffle(d, random);
                tests[i].d[0] = d.get(0);
                tests[i].d[1] = d.get(1);
                tests[i].d[2] = d.get(2);
                tests[i].d[3] = d.get(3);
            }
        }
        try (PrintWriter out = new PrintWriter(String.format("%s/%02d", root, ++nTests))) {
            System.out.println("Writing test " + nTests);
            out.println(cnt);
            for (int i = 0; i < cnt; i++) {
                out.println(tests[i].d[0] + " " + tests[i].d[1] + " " + tests[i].d[2] + " " + tests[i].d[3]);
            }
        } catch (IOException e) {
            throw new AssertionError(e);
        }
    }
}

