/******************************************************************/
/* ACM ICPC 2017-2018, NEERC                                      */
/* Northern Subregional Contest                                   */
/* St Petersburg, November 4, 2017                                */
/******************************************************************/
/* Problem F. Fygon 2.0                                           */
/*                                                                */
/* Original idea         Artem Vasilyev                           */
/* Problem statement     Artem Vasilyev                           */
/* Test set              Artem Vasilyev                           */
/******************************************************************/
/* Tests                                                          */
/*                                                                */
/* Author                Artem Vasilyev                           */
/******************************************************************/

import java.io.*;
import java.lang.reflect.Array;
import java.nio.file.*;
import java.util.*;
import java.util.stream.Collectors;

public class Tests {
    private static int nTests = 0;
    private static final Path root = Paths.get("tests");
    private static final Random random = new Random("fygon20".hashCode());

    private static final int MAX_N = 20;
    private static final int ALPH = 26;
    private static final int MAX_TESTS_N = 5;

    private static final Set<Character> ALL_VARIABLES;

    static {
        Set<Character> tmp = new HashSet<>();
        for (int i = 0; i < ALPH; i++) {
            if (i + 'a' != 'n') {
                tmp.add((char) (i + 'a'));
            }
        }
        ALL_VARIABLES = Collections.unmodifiableSet(tmp);
    }

    static class Triple {
        final char var, from, to;

        public Triple(int var, int from, int to) {
            this.var = (char) var;
            this.from = (char) from;
            this.to = (char) to;
        }


        public Triple shuffle(int[] perm) {
            return new Triple((char) (
                    'a' + perm[var - 'a']),
                    from == '1' ? '1' : (char) ('a' + perm[from - 'a']),
                    to == 'n' ? 'n' : (char) ('a' + perm[to - 'a'])
            );
        }
    }

    public static void main(String[] args) throws IOException {
        Files.createDirectories(root);
        printSample();
        printRandom(1);

        printChain(2);
        printEmpty(2);

        printChain(5);
        printEmpty(5);

        printRandom(10);
        printChain(10);
        printRandomAcyclic(10);

        printRandomAcyclic(15);
        printChain(20);
        printEmpty(20);
        printRandomAcyclic(20);
        for (int i = 0; i < MAX_TESTS_N; i++) {
            printRandom(MAX_N);
        }
        printTwoChains(3, 20);
        printTwoChains(6, 20);
        printTwoChains(9, 20);

        printCycle(2, 2);
        printCycle(2, 20);
        printCycle(5, 10);
        printCycle(20, 20);
    }

    private static void printSample() {
        printTest(Arrays.asList(
                new Triple('i', '1', 'n'),
                new Triple('j', '1', 'i'),
                new Triple('k', 'j', 'n'))
        );
    }

    private static void printRandom(int n) {
        List<Triple> triples = randomTriples(n);
        printTest(triples);
    }

    private static void printChain(int n) {
        List<Triple> triples = new ArrayList<>();
        List<Character> chain = new ArrayList<>(Arrays.asList('1', 'n'));
        for (int i = 0; i < n; i++) {
            char var = randomVariable(ALL_VARIABLES, chain);
            int pos = random.nextInt(chain.size() - 1);
            triples.add(new Triple(var, chain.get(pos), chain.get(pos + 1)));
            chain.add(pos + 1, var);
        }
        printTest(triples);
    }

    private static void printEmpty(int n) {
        List<Triple> triples = new ArrayList<>();
        Set<Character> usedVars = new HashSet<>();
        for (int i = 0; i < n; i++) {
            char var = randomVariable(ALL_VARIABLES, usedVars);
            usedVars.add(var);
            triples.add(new Triple(var, '1', 'n'));
        }
        printTest(triples);
    }

    private static void printRandomAcyclic(int n) {
        List<Triple> triples = new ArrayList<>();
        List<Character> topSort = new ArrayList<>(Arrays.asList('1', 'n'));
        for (int i = 0; i < n; i++) {
            char var = randomVariable(ALL_VARIABLES, topSort);
            int pos = random.nextInt(topSort.size() - 1);
            topSort.add(pos + 1, var);
        }
        List<Character> vars = new ArrayList<>(topSort.subList(1, topSort.size() - 1));
        Collections.shuffle(vars, random);
        Set<Character> usedVars = new HashSet<>(Arrays.asList('1', 'n'));
        for (int i = 0; i < vars.size(); i++) {
            char var = vars.get(i);
            int index = topSort.indexOf(var);
            List<Character> lower = new ArrayList<>(), higher = new ArrayList<>();
            for (int j = 0; j < topSort.size(); j++) {
                if (usedVars.contains(topSort.get(j))) {
                    if (j < index) {
                        lower.add(topSort.get(j));
                    } else {
                        higher.add(topSort.get(j));
                    }
                }
            }
            triples.add(new Triple(var, randomElement(lower), randomElement(higher)));
            usedVars.add(var);
        }
        printTest(triples);
    }


    private static void printTwoChains(int chainLength, int n) {
        List<Triple> triples = new ArrayList<>();
        List<Character> chain1 = new ArrayList<>(), chain2 = new ArrayList<>();
        Set<Character> usedVariables = new HashSet<>();

        for (int i = 0; i < chainLength; i++) {
            char var = randomVariable(ALL_VARIABLES, usedVariables);
            chain1.add(var);
            usedVariables.add(var);
            var = randomVariable(ALL_VARIABLES, usedVariables);
            chain2.add(var);
            usedVariables.add(var);
        }

        for (List<Character> list : new List[]{chain1, chain2}) {
            for (int i = 0; i < list.size(); i++) {
                triples.add(new Triple(
                        list.get(i),
                        i - 1 < 0 ? '1' : list.get(i - 1),
                        'n'
                ));
            }
        }

        for (int i = 2 * chainLength; i < n; i++) {
            char var = randomVariable(ALL_VARIABLES, usedVariables);
            usedVariables.add(var);

            int pos1 = randomRange(0, chainLength - 1);
            int pos2 = randomRange(0, chainLength - 1);
            if (pos1 > pos2) {
                int tmp = pos1;
                pos1 = pos2;
                pos2 = tmp;
            }

            triples.add(new Triple(var, chain1.get(pos1), chain2.get(pos2)));
        }
        printTest(triples);
    }

    private static void printCycle(int len, int n) {
        List<Triple> triples = new ArrayList<>();
        List<Character> chain = new ArrayList<>();
        Set<Character> usedVariables = new HashSet<>();
        char prev = '1';
        for (int i = 0; i < len - 1; i++) {
            char var = randomVariable(ALL_VARIABLES, usedVariables);
            usedVariables.add(var);
            triples.add(new Triple(var, prev, 'n'));
            chain.add(var);
            prev = var;
        }

        char looper = randomVariable(ALL_VARIABLES, usedVariables);
        usedVariables.add(looper);
        triples.add(new Triple(looper, chain.get(chain.size() - 1), chain.get(0)));

        for (int i = len; i < n; i++) {
            char var = randomVariable(ALL_VARIABLES, usedVariables);
            char varFrom = randomVariable(usedVariables, Collections.emptySet(), '1');
            char varTo = randomVariable(usedVariables, Collections.emptySet(), 'n');
            triples.add(new Triple(var, varFrom, varTo));
            usedVariables.add(var);
        }
        printTest(triples);
    }

    private static List<Triple> randomTriples(int n) {
        List<Triple> triples = new ArrayList<>();
        Set<Character> usedVariables = new HashSet<>();
        for (int i = 0; i < n; i++) {
            char var = randomVariable(ALL_VARIABLES, usedVariables);
            char varFrom = randomVariable(usedVariables, Collections.emptySet(), '1');
            char varTo = randomVariable(usedVariables, Collections.emptySet(), 'n');
            triples.add(new Triple(var, varFrom, varTo));
            usedVariables.add(var);
        }
        return triples;
    }

    private static char randomVariable(Collection<Character> include, Collection<Character> exclude, Character... includeMore) {
        List<Character> available = new ArrayList<>(include);
        available.addAll(Arrays.asList(includeMore));
        available.removeAll(exclude);
        return randomElement(available);
    }

    private static Character randomElement(List<Character> lower) {
        return lower.get(random.nextInt(lower.size()));
    }

    private static int randomRange(int from, int to) {
        return from + random.nextInt(to - from + 1);
    }

    private static List<Triple> shuffleVariables(List<Triple> triples) {
        int[] perm = randomPermutation();
        return triples
                .stream()
                .map(t -> t.shuffle(perm))
                .collect(Collectors.toList());
    }

    private static int[] randomPermutation() {
        int[] perm = new int[ALPH];
        for (int i = 0; i < perm.length; i++) {
            perm[i] = i;
            int pos = random.nextInt(i + 1);
            perm[i] = perm[pos];
            perm[pos] = i;
        }
        for (int i = 0; i < perm.length; i++) {
            if (perm[i] + 'a' == 'n') {
                int tmp = perm['n' - 'a'];
                perm['n' - 'a'] = perm[i];
                perm[i] = tmp;
                break;
            }
        }
        return perm;
    }

    private static void printTestShuffled(List<Triple> triples) {
        triples = shuffleVariables(triples);
        printTest(triples);
    }

    private static void printTest(List<Triple> triples) {
        try (PrintWriter out = new PrintWriter(String.format("%s/%02d", root, ++nTests))) {
            System.out.println("Writing test " + nTests);
            out.println(triples.size() + 1);
            for (int i = 0; i <= triples.size(); i++) {
                for (int j = 0; j < 4 * i; j++) {
                    out.print(' ');
                }
                if (i == triples.size()) {
                    out.println("lag");
                } else {
                    Triple triple = triples.get(i);
                    out.printf("for %c in range(%c, %c):%n", triple.var, triple.from, triple.to);
                }
            }
        } catch (IOException e) {
            throw new AssertionError(e);
        }
    }
}

