/******************************************************************/
/* ACM ICPC 2017-2018, NEERC                                      */
/* Northern Subregional Contest                                   */
/* St Petersburg, November 4, 2017                                */
/******************************************************************/
/* Problem F. Fygon 2.0                                           */
/*                                                                */
/* Original idea         Artem Vasilyev                           */
/* Problem statement     Artem Vasilyev                           */
/* Test set              Artem Vasilyev                           */
/******************************************************************/
/* Solution                                                       */
/*                                                                */
/* Author                Artem Vasilyev                           */
/******************************************************************/

import java.io.*;
import java.util.*;

public class fygon20_av {

    public static final int ALPH = 26;

    void solve() {
        DirectedGraph graph = readGraph();
        graph = graph.condensation();
        int k = graph.graph.length;
        long num = graph.countTopSorts();
        long denom = factorial(k);
        long gcd = gcd(num, denom);
        out.printf("%d %d/%d\n", k, num / gcd, denom / gcd);
    }

    long gcd(long a, long b) {
        if (a == 0) {
            return b;
        }
        return gcd(b % a, a);
    }

    long factorial(int k) {
        long result = 1;
        for (int i = 2; i <= k; i++) {
            result *= i;
        }
        return result;
    }

    private DirectedGraph readGraph() {
        int lines = in.nextInt();
        in.nextLine();
        DirectedGraph graph = new DirectedGraph(lines - 1);
        int[] id = new int[ALPH];
        for (int i = 0; i < lines; i++) {
            if (i == lines - 1) {
                continue;
            }
            String line = in.nextLine().trim();
            char[] vars = parseFor(line);
            id[vars[0] - 'a'] = i;
            if (vars[1] != '1') {
                graph.addEdge(id[vars[1] - 'a'], i);
            }
            if (vars[2] != 'n') {
                graph.addEdge(i, id[vars[2] - 'a']);
            }
        }
        return graph;
    }

    class DirectedGraph {
        List<Integer>[] graph;
        List<Integer>[] revGraph;

        public DirectedGraph(int n) {
            graph = new List[n];
            revGraph = new List[n];
            for (int i = 0; i < graph.length; i++) {
                graph[i] = new ArrayList<>();
                revGraph[i] = new ArrayList<>();
            }
        }

        void addEdge(int from, int to) {
            graph[from].add(to);
            revGraph[to].add(from);
        }

        List<Integer> topSort() {
            List<Integer> order = new ArrayList<>();
            boolean[] used = new boolean[graph.length];
            for (int i = 0; i < graph.length; i++) {
                if (!used[i]) {
                    dfs(i, order, used);
                }
            }
            Collections.reverse(order);
            return order;
        }

        void dfs(int u, List<Integer> order, boolean[] used) {
            used[u] = true;
            graph[u].forEach(v -> {if (!used[v]) {dfs(v, order, used);}});
            order.add(u);
        }

        DirectedGraph condensation() {
            List<Integer> topSort = topSort();
            int[] color = new int[graph.length];
            Arrays.fill(color, -1);
            int comps = 0;
            for (int u : topSort) {
                if (color[u] == -1) {
                    dfsRev(u, comps++, color);
                }
            }

            DirectedGraph result = new DirectedGraph(comps);
            for (int i = 0; i < graph.length; i++) {
                for (int j : graph[i]) {
                    if (color[i] != color[j]) {
                        result.addEdge(color[i], color[j]);
                    }
                }
            }
            return result;
        }

        void dfsRev(int u, int col, int[] color) {
            color[u] = col;
            revGraph[u].forEach(v -> {if (color[v] == -1) dfsRev(v, col, color);});
        }

        public long countTopSorts() {
            int n = graph.length;
            int[] outMask = new int[n];
            for (int i = 0; i < n; i++) {
                for (int j : graph[i]) {
                    outMask[i] |= 1 << j;
                }
            }
            long[] dp = new long[1 << n];
            dp[0] = 1;
            for (int mask = 0; mask < 1 << n; mask++) {
                for (int bit = 0; bit < n; bit++) {
                    if ((mask & (1 << bit)) == 0) {
                        if ((outMask[bit] & mask) == 0) {
                            dp[mask | (1 << bit)] += dp[mask];
                        }
                    }
                }
            }
            return dp[(1 << n) - 1];
        }
    }

    private char[] parseFor(String line) {
        char[] result = new char[3];
        line = line.substring("for ".length());
        result[0] = line.charAt(0);
        line = line.substring("i in range(".length());
        result[1] = line.charAt(0);
        line = line.substring("1, ".length());
        result[2] = line.charAt(0);
        return result;
    }

    public static final String FILENAME = "fygon20";

    void run() throws FileNotFoundException {
        in = new Scanner(new File(FILENAME + ".in"));
        out = new PrintWriter(new File(FILENAME + ".out"));
        solve();
        out.close();
    }

    Scanner in;
    PrintWriter out;

    public static void main(String[] args) throws FileNotFoundException {
        new fygon20_av().run();
    }
}
