/******************************************************************/
/* ACM ICPC 2017-2018, NEERC                                      */
/* Northern Subregional Contest                                   */
/* St Petersburg, November 4, 2017                                */
/******************************************************************/
/* Problem G. Grand Test                                          */
/*                                                                */
/* Original idea         Andrew Stankevich                        */
/* Problem statement     Borys Minaiev                            */
/* Test set              Borys Minaiev                            */
/******************************************************************/
/* Solution                                                       */
/*                                                                */
/* Author                Borys Minaiev                            */
/******************************************************************/

import java.io.*;
import java.lang.reflect.Array;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.StringTokenizer;

public class grand_bm {
    private FastScanner in;
    private PrintWriter out;

    private void solve() {
        int tc = in.nextInt();
        for (int t = 0; t < tc; t++) {
            int n = in.nextInt();
            int m = in.nextInt();
            ArrayList<Integer>[] g = new ArrayList[n];
            for (int i = 0; i < n; i++) {
                g[i] = new ArrayList<>();
            }
            for (int i = 0; i < m; i++) {
                int fr = in.nextInt() - 1;
                int to = in.nextInt() - 1;
                g[fr].add(to);
                g[to].add(fr);
            }
            ArrayList<ArrayList<Integer>> answer = solve(g);
            if (answer == null) {
                out.println(-1);
            } else {
                ArrayList<Integer> path = answer.get(0);
                out.println((path.get(0) + 1) + " " + (path.get(path.size() - 1) + 1));
                for (ArrayList<Integer> p : answer) {
                    out.print(p.size());
                    for (int x : p) {
                        out.print(" " + (x + 1));
                    }
                    out.println();
                }
            }
        }
    }

    class Edge {
        int fr, to;

        public Edge(int fr, int to) {
            this.fr = fr;
            this.to = to;
        }
    }

    ArrayList<ArrayList<Integer>> solve(ArrayList<Integer>[] g) {
        int n = g.length;
        int[] was = new int[n];
        Edge[] edges = new Edge[n];
        int[] parent = new int[n];
        for (int i = 0; i < n; i++) {
//            System.err.println(i + "/ " + n);
            if (was[i] != 0) {
                continue;
            }
            Edge[] answer = go(i, i, was, edges, parent, g);
//            System.err.println(Arrays.toString(answer));
            if (answer != null) {
                return buildAnswer(answer, n, parent);
            }
        }
        return null;
    }

    private ArrayList<ArrayList<Integer>> buildAnswer(Edge[] edges, int n, int[] parent) {
        ArrayList<Integer>[] g = new ArrayList[n];
        for (int i = 0; i < g.length; i++) {
            g[i] = new ArrayList<>();
        }
        for (Edge e : edges) {
            g[e.fr].add(e.to);
            g[e.to].add(e.fr);
            for (int x = e.fr; x != e.to; x = parent[x]) {
                if (g[x].contains(parent[x])) {
                    continue;
                }
                g[x].add(parent[x]);
                g[parent[x]].add(x);
            }
        }
        ArrayList<Integer> roots = new ArrayList<>();
        for (int i = 0; i < n; i++) {
            if (g[i].size() == 3) {
                roots.add(i);
            }
        }
        assert roots.size() == 2;
        ArrayList<ArrayList<Integer>> result = new ArrayList<>();
        dfs(roots.get(0), roots.get(0), roots.get(1), g, result, new ArrayList<Integer>());
        assert result.size() == 3;
        return result;
    }

    private void dfs(int v, int p, int need, ArrayList<Integer>[] g, ArrayList<ArrayList<Integer>> answer, ArrayList<Integer> current) {
        current.add(v);
        if (v == need) {
            answer.add((ArrayList<Integer>) current.clone());
        } else {
            for (int to : g[v]) {
                if (to == p) {
                    continue;
                }
                dfs(to, v, need, g, answer, current);
            }
        }
        current.remove(current.size() - 1);
    }


    Edge[] go(int v, int p, int[] was, Edge[] edges, int[] parent, ArrayList<Integer>[] g) {
        was[v] = 1;
//        System.err.println("go " + v + " " + p);
        parent[v] = p;
        for (int to : g[v]) {
//            System.err.println("edge " + v + " -> " + to);
            if (to == p || was[to] == 2) {
                continue;
            }
            if (was[to] == 1) {
                Edge e = new Edge(v, to);
//                System.err.println("find edge " + v +" " + to);
                Edge e2 = null;
                for (int x = v; x != to; x = parent[x]) {
//                    System.err.println("x = " + x);
                    if (edges[x] != null) {
                        e2 = edges[x];
                    }
                    edges[x] = e;
                }
                if (e2 != null) {
                    return new Edge[]{e, e2};
                }
            } else {
                Edge[] res = go(to, v, was, edges, parent, g);
                if (res != null) {
                    return res;
                }
            }
        }
        was[v] = 2;
        return null;
    }


    private void run() {
        try {
            in = new FastScanner(new File("grand.in"));
            out = new PrintWriter(new File("grand.out"));

            solve();

            out.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
    }

    private class FastScanner {
        BufferedReader br;
        StringTokenizer st;

        FastScanner(File f) {
            try {
                br = new BufferedReader(new FileReader(f));
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            }
        }

        FastScanner(InputStream f) {
            br = new BufferedReader(new InputStreamReader(f));
        }

        String next() {
            while (st == null || !st.hasMoreTokens()) {
                String s = null;
                try {
                    s = br.readLine();
                } catch (IOException e) {
                    e.printStackTrace();
                }
                if (s == null)
                    return null;
                st = new StringTokenizer(s);
            }
            return st.nextToken();
        }

        boolean hasMoreTokens() {
            while (st == null || !st.hasMoreTokens()) {
                String s = null;
                try {
                    s = br.readLine();
                } catch (IOException e) {
                    e.printStackTrace();
                }
                if (s == null)
                    return false;
                st = new StringTokenizer(s);
            }
            return true;
        }

        int nextInt() {
            return Integer.parseInt(next());
        }

        long nextLong() {
            return Long.parseLong(next());
        }

        double nextDouble() {
            return Double.parseDouble(next());
        }
    }

    public static void main(String[] args) {
        new grand_bm().run();
    }
}
