/******************************************************************/
/* ACM ICPC 2017-2018, NEERC                                      */
/* Northern Subregional Contest                                   */
/* St Petersburg, November 4, 2017                                */
/******************************************************************/
/* Problem G. Grand Test                                          */
/*                                                                */
/* Original idea         Andrew Stankevich                        */
/* Problem statement     Borys Minaiev                            */
/* Test set              Borys Minaiev                            */
/******************************************************************/
/* Solution                                                       */
/*                                                                */
/* Author                Georgiy Korneev                          */
/******************************************************************/

import java.io.*;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public class grand_gk {
    static Scanner in;
    static PrintWriter out;

    static class Solution {
        List<List<Integer>> edges;
        int time;
        boolean[] visited;
        boolean[] visited2;
        int[] tin, up;
        Deque<Edge> stack = new ArrayDeque<>();
        private Map<Integer, Integer> indices;

        void run() {
            int n = in.nextInt();
            int m = in.nextInt();
//            System.err.println("n, m = " + n + " " + m);
            edges = Stream.generate(ArrayList<Integer>::new).limit(n).collect(Collectors.toList());
            for (int i = 0; i < m; i++) {
                int u = in.nextInt() - 1;
                int v = in.nextInt() - 1;
                edges.get(u).add(v);
                edges.get(v).add(u);
            }

            try {
                visited = new boolean[n];
                visited2 = new boolean[n];
                tin = new int[n];
                up = new int[n];
                for (int v = 0; v < n; v++) {
                    if (!visited[v]) {
                        paint(v, -1);
                    }
                }
                out.println(-1);
            } catch (ResultException e) {
                final List<List<Integer>> paths = e.paths;
                final List<Integer> f = paths.get(0);
                out.println(f.get(0) + " " + f.get(f.size() - 1));
                for (List<Integer> path : paths) {
                    out.println(path.size() + " " + path.stream().map(Object::toString).collect(Collectors.joining(" ")));
                }
            }
        }

        void paint(final int v, final int parent) {
            visited[v] = true;
//            System.err.println("in " + (v + 1));
            tin[v] = up[v] = ++time;
            for (int u : edges.get(v)) {
                if (u == parent) {
                    continue;
                }
                if (!visited[u]) {
                    final Edge edge = new Edge(v, u);
                    stack.push(edge);
                    paint(u, v);
                    if (up[u] >= tin[v]) {
                        List<Edge> block = new ArrayList<>();
                        while (stack.peek() != edge) {
                            block.add(stack.pop());
                        }
                        block.add(stack.pop());
                        check(block);
                    }
                    if (up[v] > up[u]) {
                        up[v] = up[u];
                    }
                } else if (tin[v] > tin[u]) {
                    stack.push(new Edge(v, u));
                    up[v] = Math.min(up[v], tin[u]);
                } else if (up[v] > tin[u]){
                    up[v] = tin[u];
                }
            }
//            System.err.println("out " + (v + 1));
        }

        private void check(final List<Edge> block) {
            indices = new HashMap<>();
            for (Edge edge : block) {
                indices.merge(edge.u, 1, Integer::sum);
                indices.merge(edge.v, 1, Integer::sum);
            }
//            System.err.println("b: " + block);
//            System.err.println("vs: " + Arrays.toString(indices.keySet().stream().sorted().mapToInt(i -> i + 1).toArray()));
            indices.entrySet().stream()
                    .filter(e -> e.getValue() > 2)
                    .map(Map.Entry::getKey)
                    .findFirst().ifPresent(v -> {
//                System.err.println("v: " + (v + 1));
//                        List<List<Integer>> paths = dfs(v, -1, v);
                Arrays.fill(visited, false);
                final List<Integer> cycle = findCycle(v, -1, v);
                cycle.add(v + 1);
//                System.err.println(cycle);
                final List<Integer> path1 = findPath(v, -1, v);
                path1.add(v + 1);
                final int t = path1.get(0);
                final List<Integer> path2 = path(cycle, t);
                Collections.reverse(cycle);
                final List<Integer> path3 = path(cycle, t);
                Collections.reverse(path1);
//                System.err.println("1: " + path1);
//                System.err.println("2: " + path2);
//                System.err.println("3: " + path3);
//                System.err.println(edges.size());
                assert path1.size() > 1;
                assert path2.size() > 1;
                assert path3.size() > 1;
                throw new ResultException(Arrays.asList(path1, path2, path3));
            });
        }

        private List<Integer> path(final List<Integer> cycle, final int t) {
            final List<Integer> path2 = new ArrayList<>();
            for (int u : cycle) {
                path2.add(u);
                if (u == t) {
                    break;
                }
            }
            return path2;
        }

        private List<Integer> findCycle(final int v, int parent, final int t) {
            visited[v] = true;

            for (int u : edges.get(v)) {
                if (u == parent) {
                    continue;
                }
                if (u == t) {
                    edges.get(v).remove((Object) u);
                    edges.get(u).remove((Object) v);
//                    System.out.println(u + 1);
                    return new ArrayList<>(Collections.singletonList(u + 1));
                } else if (!visited[u] && indices.containsKey(u)){
                    List<Integer> cycle = findCycle(u, v, t);
                    if (cycle != null) {
                        cycle.add(u + 1);
                        edges.get(v).remove((Object) u);
                        edges.get(u).remove((Object) v);
                        return cycle;
                    }
                }
            }
            return null;
        }

        private List<Integer> findPath(final int v, final int parent, final int source) {
            visited2[v] = true;

            for (int u : edges.get(v)) {
                if (u == parent) {
                    continue;
                }
                if (u == source) {
                    continue;
                }
                if (visited[u]) {
                    return new ArrayList<>(Collections.singletonList(u + 1));
                } else if (!visited2[u] && indices.containsKey(u)) {
                    List<Integer> path = findPath(u, v, source);
                    if (path != null) {
                        path.add(u + 1);
                        return path;
                    }
                }
            }
            return null;
        }
    }

    public static void main(String[] args) throws Exception {
        in = new Scanner(new File("grand.in"));
        out = new PrintWriter("grand.out");

        long START = System.currentTimeMillis();

        int t = in.nextInt();
        for (int i = 0; i < t; i++) {
//            System.err.println(i + " ---");
            new Solution().run();
//            out.flush();
        }

        System.err.println(System.currentTimeMillis() - START);

        in.close();
        out.close();
    }

    static class Scanner {
        final BufferedReader reader;
        StringTokenizer tokenizer = new StringTokenizer("");

        Scanner(File file) throws FileNotFoundException {
            reader = new BufferedReader(new FileReader(file));
        }

        int nextInt() {
            return Integer.parseInt(next());
        }

        String next() {
            while (!tokenizer.hasMoreTokens()) {
                try {
                    tokenizer = new StringTokenizer(reader.readLine());
                } catch (IOException e) {
                    throw new AssertionError();
                }
            }
            return tokenizer.nextToken();
        }

        void close() {
        }
    }

    static class Edge {
        final int u, v;

        Edge(final int u, final int v) {
            this.u = u;
            this.v = v;
        }

        @Override
        public String toString() {
            return "(" + (u + 1) + ", " + (v + 1) + ")";
        }
    }

    static class ResultException extends RuntimeException {
        List<List<Integer>> paths;

        ResultException(final List<List<Integer>> paths) {
            this.paths = paths;
        }
    }
}

