/******************************************************************/
/* ACM ICPC 2017-2018, NEERC                                      */
/* Northern Subregional Contest                                   */
/* St Petersburg, November 4, 2017                                */
/******************************************************************/
/* Problem H. Hidden Supervisors                                  */
/*                                                                */
/* Original idea         Niyaz Nigmatullin                        */
/* Problem statement     Niyaz Nigmatullin                        */
/* Test set              Niyaz Nigmatullin                        */
/******************************************************************/
/* Solution                                                       */
/*                                                                */
/* Author                Niyaz Nigmatullin                        */
/******************************************************************/

import java.io.BufferedReader
import java.io.File
import java.io.FileReader
import java.io.PrintWriter
import java.util.*

private class SolutionNNKt2(val sc: FastScanner) {

    class FastScanner(file: File) {
        var reader = BufferedReader(FileReader(file))
        var tokenizer = StringTokenizer("")

        fun nextInt() = next().toInt()

        private fun next(): String {
            while (!tokenizer.hasMoreTokens()) {
                tokenizer = StringTokenizer(reader.readLine())
            }
            return tokenizer.nextToken()
        }

    }

    val n = sc.nextInt()
    val p = IntArray(n) { if (it == 0) -1 else sc.nextInt() - 1 }
    val deg = with(p) {
        val d = IntArray(n)
        for (i in 1 until n) {
            if (p[i] >= 0) {
                d[p[i]]++
            }
        }
        d
    }
    val edges = with(p) {
        val edges = Array(n) { IntArray(deg[it]) }
        val cnt = IntArray(n)
        for (i in 1 until n) {
            val parent = p[i]
            if (parent >= 0) {
                edges[parent][cnt[parent]++] = i
            }
        }
        edges
    }
    val mSize = IntArray(n)
    val freeSize = IntArray(n)
    val freeVertices = IntArray(n)
    var cntFree = 0
    val isFree = BooleanArray(n) { true }
    val q = IntArray(n)

    fun addFree(start: Int) {
        var head = 0
        var tail = 0
        q[tail++] = start
        while (head < tail) {
            val v = q[head++]
            if (isFree[v]) {
                freeVertices[cntFree++] = v
            }
            for (to in edges[v]) {
                q[tail++] = to
            }
        }
    }

    fun solve() {
        val answer = p.clone()
        val freeRoots = arrayListOf<Pair<Int, Int>>()
        var head = 0
        var tail = 0
        for (i in 0 until n) {
            if (deg[i] == 0) {
                q[tail++] = i
            }
        }
        while (head < tail) {
            val v = q[head++]
            val parent = p[v]
            if (parent < 0) continue
            --deg[parent]
            mSize[parent] += mSize[v]
            freeSize[parent] += freeSize[v] + 1
            if (isFree[v] && isFree[parent]) {
                isFree[parent] = false
                isFree[v] = false
                mSize[parent]++
                freeSize[parent] -= 2
            }
            if (deg[parent] == 0) {
                q[tail++] = parent
            }
        }
        var ans = 0
        for (v in 0 until n) {
            if (p[v] >= 0) continue
            ans += mSize[v]
            if (v == 0 || !isFree[v]) {
                addFree(v)
                answer[v] = 0
            } else {
                freeRoots.add(Pair(freeSize[v], v))
            }
        }
        Collections.sort(freeRoots, compareBy { -it.first })
        for (cur in freeRoots) {
            val v = cur.second
            if (cntFree > 0) {
                ans++
                answer[v] = freeVertices[--cntFree]
                isFree[v] = false
            } else {
                answer[v] = 0
            }
            addFree(v)
        }
        PrintWriter("hidden.out").use {
            with(it) {
                println(ans)
                println(answer.drop(1).joinToString(" ", transform = { (it + 1).toString() }))
            }
        }
    }
}

fun main(args: Array<String>) {
    SolutionNNKt2(SolutionNNKt2.FastScanner(File("hidden.in"))).solve()
}

