/******************************************************************/
/* ACM ICPC 2017-2018, NEERC                                      */
/* Northern Subregional Contest                                   */
/* St Petersburg, November 4, 2017                                */
/******************************************************************/
/* Problem J. Joker                                               */
/*                                                                */
/* Original idea         Gennady Korotkevich                      */
/* Problem statement     Borys Minaiev                            */
/* Test set              Borys Minaiev                            */
/******************************************************************/
/* Solution                                                       */
/*                                                                */
/* Author                Borys Minaiev                            */
/******************************************************************/

import java.io.*;
import java.util.*;

public class joker_bm {
    private FastScanner in;
    private PrintWriter out;

    private void solve() {
        long START = System.currentTimeMillis();
        int n = in.nextInt();
        int m = in.nextInt();
        int[] a = new int[n];
        for (int i = 0; i < n; i++) {
            a[i] = in.nextInt();
        }
        int[] pos = new int[m];
        int[] val = new int[m];
        for (int i = 0; i < m; i++) {
            pos[i] = in.nextInt() - 1;
            val[i] = in.nextInt();
        }
        int[] res = solveCorrect(a, pos, val);
        for (int i : res) {
            out.println(i + 1);
        }
        System.err.println("done in " + (System.currentTimeMillis() - START) + " ms");
    }


    private int vectMul(int a, int b, int c) {
        return Long.signum((sumPositivePt[b] - sumPositivePt[a]) * (sumNegativePt[c] - sumNegativePt[a]) -
                (sumNegativePt[b] - sumNegativePt[a]) * (sumPositivePt[c] - sumPositivePt[a]));
    }

    private long[] sumPositivePt;
    private long[] sumNegativePt;
    private int[] realPosPt;

    private class Block {
        int from;
        int size;
        int sumPositive;
        int sumNegative;

        void setNewValue(int value) {
            if (value > 0) {
                sumPositive = value;
                sumNegative = 0;
            } else {
                sumPositive = 0;
                sumNegative = -value;
            }
            sumNegativePt[from] = sumNegative;
            sumPositivePt[from] = sumPositive;
        }

        int bestPos = -1;
        int prevSumPos, prevSumNeg, totalSumNeg, totalSumPos;

        long calc(int i) {
            return (prevSumPos + sumPositivePt[i]) * totalSumNeg - (prevSumNeg + sumNegativePt[i]) * totalSumPos;
        }

        long findBestAns(int prevSumPos, int prevSumNeg, int totalSumPos, int totalSumNeg) {
            this.prevSumNeg = prevSumNeg;
            this.prevSumPos = prevSumPos;
            this.totalSumNeg = totalSumNeg;
            this.totalSumPos = totalSumPos;
            int l = from, r = from + size;
            while (r - l > 2) {
                int mid = (l + r) >> 1;
                if (calc(mid) < calc(mid + 1)) {
                    l = mid;
                } else {
                    r = mid + 1;
                }
            }
            long res = Long.MIN_VALUE;
            for (int i = l; i < r; i++) {
                long value = calc(i);
                if (value > res) {
                    bestPos = realPosPt[i];
                    res = value;
                }
            }
            return res;
        }

        void build(int from, int to) {
            this.from = from;
            size = 0;
            for (int i = from; i < to; i++) {
                if (size > 0 && sumPositivePt[from + size - 1] == sumPositivePt[i]) {
                    continue;
                }
                while (size >= 2) {
                    if (vectMul(from + size - 2, from + size - 1, i) > 0) {
                        break;
                    } else {
                        size--;
                    }
                }
                if (size >= 1 && sumNegativePt[from] == sumNegativePt[i]) {
                    size--;
                }
                sumPositivePt[from + size] = sumPositivePt[i];
                sumNegativePt[from + size] = sumNegativePt[i];
                realPosPt[from + size] = i;
                size++;
            }

        }

        Block() {

        }

        void create(int[] a, int from, int to) {
            sumPositive = 0;
            sumNegative = 0;
            for (int i = from; i < to; i++) {
                if (a[i] > 0) {
                    sumPositive += a[i];
                } else {
                    sumNegative -= a[i];
                }
                sumNegativePt[i] = sumNegative;
                sumPositivePt[i] = sumPositive;

            }
            build(from, to);
        }
    }


    private int findBestAns(Block[] blocks, int blocksSz, int totalSumPos, int totalSumNeg) {
        int curSumPos = 0, curSumNeg = 0;
        long result = Long.MIN_VALUE;
        int bestPos = -1;
        for (int i = 0; i < blocksSz; i++) {
            Block b = blocks[i];
            long res = b.findBestAns(curSumPos, curSumNeg, totalSumPos, totalSumNeg);
            if (res > result) {
                result = res;
                bestPos = b.bestPos;
            }
            curSumPos += b.sumPositive;
            curSumNeg += b.sumNegative;
        }
        return bestPos;
    }

    private int[] solveCorrect(int[] a, int[] pos, int[] val) {
        final int changes = pos.length;
        final int n = a.length;
        int[] res = new int[changes + 1];
        final int SQRT = 500;
        boolean[] toBeChanged = new boolean[n];
        sumNegativePt = new long[n];
        sumPositivePt = new long[n];
        realPosPt = new int[n];
        Block[] blocks = new Block[2 * SQRT + 3];
        for (int i = 0; i < blocks.length; i++) {
            blocks[i] = new Block();
        }
        int sumPositive = 0;
        int sumNegative = 0;
        for (int i = 0; i < n; i++) {
            if (a[i] > 0) {
                sumPositive += a[i];
            } else {
                sumNegative -= a[i];
            }
        }
        for (int it = 0; it < changes; it += SQRT) {
            Arrays.fill(toBeChanged, false);
            for (int i = it; i < it + SQRT && i < changes; i++) {
                toBeChanged[pos[i]] = true;
            }
            int blocksSz = 0;
            int convexSz = 0;
            for (int i = 0; i < n; ) {
                int j = i + 1;
                if (!toBeChanged[i]) {
                    while (j != n && !toBeChanged[j]) {
                        j++;
                    }
                }
                blocks[blocksSz++].create(a, i, j);
                i = j;
                convexSz += blocks[blocksSz - 1].size;
            }
//            System.err.println("convex = " + convexSz);
            if (it == 0) {
                res[0] = findBestAns(blocks, blocksSz, sumPositive, sumNegative);
            }
            for (int i = it; i < it + SQRT && i < changes; i++) {
                int posi = pos[i];
                int nval = val[i];
                if (a[posi] > 0) {
                    sumPositive -= a[posi];
                } else {
                    sumNegative += a[posi];
                }
                a[posi] = nval;
                if (nval > 0) {
                    sumPositive += nval;
                } else {
                    sumNegative -= nval;
                }
                for (int j = 0; j < blocksSz; j++) {
                    if (blocks[j].from == posi) {
                        blocks[j].setNewValue(nval);
                        break;
                    }
                }
                res[i + 1] = findBestAns(blocks, blocksSz, sumPositive, sumNegative);
            }

        }
        return res;
    }

    private void run() {
        try {
            in = new FastScanner(new File("joker.in"));
            out = new PrintWriter(new File("joker.out"));

            solve();

            out.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
    }

    private void runIO() {
        in = new FastScanner(System.in);
        out = new PrintWriter(System.out);

        solve();

        out.close();
    }

    private class FastScanner {
        BufferedReader br;
        StringTokenizer st;

        FastScanner(File f) {
            try {
                br = new BufferedReader(new FileReader(f));
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            }
        }

        FastScanner(InputStream f) {
            br = new BufferedReader(new InputStreamReader(f));
        }

        String next() {
            while (st == null || !st.hasMoreTokens()) {
                String s = null;
                try {
                    s = br.readLine();
                } catch (IOException e) {
                    e.printStackTrace();
                }
                if (s == null)
                    return null;
                st = new StringTokenizer(s);
            }
            return st.nextToken();
        }

        boolean hasMoreTokens() {
            while (st == null || !st.hasMoreTokens()) {
                String s = null;
                try {
                    s = br.readLine();
                } catch (IOException e) {
                    e.printStackTrace();
                }
                if (s == null)
                    return false;
                st = new StringTokenizer(s);
            }
            return true;
        }

        int nextInt() {
            return Integer.parseInt(next());
        }

        long nextLong() {
            return Long.parseLong(next());
        }

        double nextDouble() {
            return Double.parseDouble(next());
        }
    }

    public static void main(String[] args) {
        new joker_bm().run();
    }
}
